from xpanse.base_api import ExApi
from xpanse.api.services.v1.classifications import ClassificationsEndpoint
from xpanse.api.services.v1.country_codes import CountryCodesEndpoint
from xpanse.api.services.v1.services import ServicesEndpoint


class ClassificationsAPI(ExApi, ClassificationsEndpoint):
    def __init__(self, session):
        super().__init__(session)

    @property
    def current_version(self):
        return "v1"

    @property
    def v1(self):
        return ClassificationsEndpoint(self._api)


class CountryCodesAPI(ExApi, CountryCodesEndpoint):
    def __init__(self, session):
        super().__init__(session)

    @property
    def current_version(self):
        return "v1"

    @property
    def v1(self):
        return CountryCodesEndpoint(self._api)


class ServicesAPI(ExApi, ServicesEndpoint):
    def __init__(self, session):
        super().__init__(session)

    @property
    def current_version(self):
        return "v1"

    @property
    def v1(self):
        return ServicesEndpoint(self._api)


class ServicesApi(ExApi):
    @property
    def classifications(self):
        return ClassificationsAPI(self._api)

    @property
    def country_codes(self):
        return CountryCodesAPI(self._api)

    @property
    def services(self):
        return ServicesAPI(self._api)



from csv import reader

import click
from xpanse.client import ExClient




def fetch_tags(client):
    return client.annotations.tags.list(disabled=False).dump()


def create_tag(client, tag):
    print(f"Creating a new tag named {tag}")
    return client.annotations.tags.create(name=tag)["id"]


def build_tag_id_list(client, tags, tag_map):
    id_list = []
    for tag in tags.split("|"):
        tag = tag.strip()
        if tag in tag_map:
            id_list.append(tag_map[tag])
        else:
            new_tag = create_tag(client, tag)
            tag_map[tag] = new_tag
            id_list.append(new_tag)
    return id_list, tag_map


def assign_tags(client, assetType, assetKey, tags, tag_map, operation):
    """
    Create a bulk assignment request and send to the appropriate endpoint.
    """
    assetID = None
    if assetType == "domain":
        assetID = client.assets.domains.get(name=assetKey).get("id")
    elif assetType == "certificate":
        assetID = client.assets.certificates.get(pemMd5Hash=assetKey).get("id")
    elif assetType == "cloud-resource":
        asseteID == client.assets.cloud_resources.get(cloudResourceId=assetKey).get("id")

    tag_list, tag_map = build_tag_id_list(client, tags, tag_map)

    if assetID is not None:
        print(
            f"Applying {len(tag_list)} tags to {assetType} asset {assetKey} with id {assetID}"
        )
        if assetType == "domain":
            client.assets.domains.bulk_tag(
                operation=operation, asset_ids=[assetID], tag_ids=tag_list
            )
        if assetType == "certificate":
            client.assets.certificates.bulk_tag(
                operation=operation, asset_ids=[assetID], tag_ids=tag_list
            )
        if assetType == "cloud-resource":
            client.assets.cloud_resources.bulk_tag(
                operation=operation, asset_ids=[assetID], tag_ids=tag_list
            )

    return tag_map


@click.command()
@click.argument('file_name', type=click.File('r'))
def cli(file_name):

    # Initialize the Xpanse Client
    client = ExClient()

    # Populate our tag map from name to id
    tag_map = {tag["name"]: tag["id"] for tag in fetch_tags(client)}

    # Wrap the input file in a csv reader
    csvreader = reader(file_name)

    # Skip the header column
    next(csvreader)

    # For row of input do a bulk assignment request
    for row in csvreader:
        assetType, assetKey, tagNames, operation = row
        tag_map = assign_tags(
            client=client,
            assetType=assetType.strip(),
            assetKey=assetKey.strip(),
            tags=tagNames,
            tag_map=tag_map,
            operation=operation,
        )


if __name__ == "__main__":
    cli()



import logging
from typing import Optional

from requests import Response


class XpanseException(Exception):
    """
    Base exception class.
    """

    def __init__(self, msg: str, response: Response = None):
        self._log = logging.getLogger(
            "{}.{}".format(self.__module__, self.__class__.__name__)
        )
        self.msg = str(msg)
        self.resp = response
        self._log.error(self.msg)

    def __str__(self) -> str:
        return self.msg

    def __repr__(self) -> str:
        return repr(self.__str__())


class UnexpectedValueError(XpanseException):
    """
    An unexpected value error is thrown whenever the value specified for a
    parameter is outside the bounds of what is expected or is missing.
    """

    pass


class UnexpectedResponseError(XpanseException):
    """
    Response from the server was unexpected and requires further investigation.
    """

    pass


class JWTExpiredError(XpanseException):
    """
    JWT has expired.
    """

    pass



from typing import Any, Dict, List

from xpanse.const import V2_PREFIX
from xpanse.endpoint import ExEndpoint
from xpanse.iterator import ExResultIterator


class IpRangeEndpoint(ExEndpoint):
    """
    Part of the Assets v2 API for handling IP Ranges.
    See: https://api.expander.expanse.co/api/v1/docs/
    """

    def list(self, **kwargs: Any) -> ExResultIterator:
        """
        Returns the list of IP Ranges. Arguments should be passed as keyword args using
        the names below.

        Args:
            limit (int, optional):
                Returns at most this many results in a single api call.
                Default is 100, max is 10000.
            offset (int, optional):
                Returns results starting at this offset.
                Default is 0.
            sort (str, optional):
                Comma-separated string; orders results by the given fields. If the field name is
                prefixed by a -, then the ordering will be descending for that field.
                Use a dotted notation to order by fields that are nested.
            business_units (str, optional):
                Comma-separated string; Returns only results whose Business Unit's ID falls in the provided list.
                NOTE: If omitted, API will return results for all Business Units the user has permissions to view.
                Also, cannot be used with the business-unit-names parameter.
            business_unit_names (str, optional):
                Comma-separated string; Returns only results whose Business Unit's name falls in the provided list.
                NOTE: If omitted, API will return results for all Business Units the user has permissions to view.
                Also, cannot be used with the business-units parameter.
            inet (str, optional):
                Search for given IP/CIDR block using a single IP (d.d.d.d), a dashed IP range (d.d.d.d-d.d.d.d),
                a CIDR block (d.d.d.d/m), a partial CIDR (d.d.), or a wildcard (d.d.*.d).
                Returns only results whose [startAddress, endAddress] range overlap with the given IP Address or CIDR.
            tags (str, optional):
                Comma-separated string; Returns only results who are associated with the provided Tag IDs.
                Cannot be used with the tag-names parameter.
            tag_names (str, optional):
                Comma-separated string; Returns only results who are associated with the provided Tag names.
                Cannot be used with the tags parameter.
            include (str, optional):
                Comma-separated string; Include the provided fields as part of the serialized result. Allowed values are
                `annotations`, `severityCounts`, `attributionReasons`, `relatedRegistrationInformation`, `certDetails`, and `locationInformation`

        Returns:
            :obj:`ExResultIterator`:
                An iterator containing all of the ip_range results. Results can be iterated
                or called by page using `<iterator>.next()`.

        Examples:
            >>> # Return all ip ranges and print each range:
            >>> for res in client.assets.ip_range.v2.list():
            ...     for ip_r in res:
            ...         print(ip_r)
        """
        return ExResultIterator(self._api, f"{V2_PREFIX}/ip-range", kwargs)

    def get(self, id: str, **kwargs: Any) -> Dict[str, Any]:
        """
        Returns the details for a given IP Range. Arguments should be passed as keyword args using
        the names below.

        Args:
            id (str):
                ID for the ip-range. Should be a UUID.
            include (str, optional):
                Comma-separated string; Include the provided fields as part of the serialized result.

        Returns:
            :obj:`dict`:
                A dictionary containing all of the details about an IP Range.

        Examples:
            >>> # Return IP Range with severity counts
            >>> my_range = client.assets.ip_range.v2.get(<id>, include="severityCounts")
        """
        return self._api.get(f"{V2_PREFIX}/ip-range/{id}", params=kwargs).json()

    def create(
        self, startAddress: str, endAddress: str, parentId: str, **kwargs: Any
    ) -> Dict[str, Any]:
        """
        Creates a new custom IP Range.
        NOTE: A validation error will be returned if the start and end addresses of the custom range do not fit within a top level range defined by Xpanse.

        Args:
            startAddress (str):
                Start address of custom ip-range.
            endAddress (str):
                End address of custom ip-range.
            parentId (str):
                Id of parent ip-range.
            tags (list, optional):
                A list of tag annotation names.
            additionalNotes (str, optional):
                Any additional notes about the custom ip-range.
            pointOfContactIds (list, optional):
                A lost of point-of-contact annotation ids.

        Returns:
            :obj:`dict`:
                A dictionary containing all of the details about the newly created, custom IP Range.

        Examples:
            >>> # Create a new ip-range under a parent range
            >>> new_range = client.assets.ip_range.v2.create("12.175.114.120", "12.175.114.121", "43a5a569-27b0-39b5-98f4-22b9885546d7", additionalNotes="Business Unit X - Marketing website hosts")
        """
        payload: Dict[str, Any] = {
            "startAddress": startAddress,
            "endAddress": endAddress,
            "parentId": parentId,
            "annotations": {},
        }

        if "tags" in kwargs:
            payload["annotations"]["tags"] = kwargs["tags"]
        if "additionalNotes" in kwargs:
            payload["annotations"]["additionalNotes"] = kwargs["additionalNotes"]
        if "pointOfContactIds" in kwargs:
            payload["annotations"]["pointOfContactIds"] = kwargs["pointOfContactIds"]
        return self._api.post(f"{V2_PREFIX}/ip-range", json=payload).json()

    def delete(self, id: str) -> bool:
        """
        Delete the given IP Range, and all connections to other data.
        NOTE: This will only work for user-defined IP Ranges.

        Args:
            id (str):
                ID for the ip-range. Should be a UUID.

        Returns:
            :obj:`boolean`:
                `True` if the range was successfully deleted, otherwise `False`.

        Examples:
            >>> # Deletes a user defined range
            >>> client.assets.ip_range.v2.delete("43a5a569-27b0-39b5-98f4-22b9885546d7")
        """
        return (
            True
            if self._api.delete(f"{V2_PREFIX}/ip-range/{id}").status_code == 204
            else False
        )

    def update(self, id: str, **kwargs: Any) -> Dict[str, Any]:
        """
        Allows the partial update of the given IP Range.

        Args:
            id (str):
                ID for the ip-range. Should be a UUID.
            startAddress (str, optional):
                Start address of custom ip-range.
            endAddress (str, optional):
                End address of custom ip-range.
            parentId (str, optional):
                Id of parent ip-range.
            tags (list, optional):
                A list of tag annotation ids.
            additionalNotes (str, optional):
                Any additional notes about the custom ip-range.
            pointOfContactIds (list, optional):
                A lost of point-of-contact annotation ids.

        Returns:
            :obj:`dict`:
                A dictionary containing all of the details about the updated, custom IP Range.

        Examples:
            >>> # Update an ip-range under a parent range
            >>> new_range = client.assets.ip_range.v2.update("43a5a569-27b0-39b5-98f4-22b9885546d7", additionalNotes="Business Unit X - Development Environment")
        """
        payload = {}
        if "startAddress" in kwargs:
            payload["startAddress"] = kwargs["startAddress"]
        if "endAddress" in kwargs:
            payload["endAddress"] = kwargs["endAddress"]
        if "parentId" in kwargs:
            payload["parentId"] = kwargs["parentId"]

        if any(
            arg in ("tags", "additionalNotes", "pointOfContactIds") for arg in kwargs
        ):
            payload["annotations"] = {}
        if "tags" in kwargs:
            payload["annotations"]["tags"] = kwargs["tags"]
        if "additionalNotes" in kwargs:
            payload["annotations"]["additionalNotes"] = kwargs["additionalNotes"]
        if "pointOfContactIds" in kwargs:
            payload["annotations"]["pointOfContactIds"] = kwargs["pointOfContactIds"]
        return self._api.patch(f"{V2_PREFIX}/ip-range/{id}", json=payload).json()

    def tag(self, ranges: List[str], tags: List[str]) -> bool:
        """
        Adds the provided tags to all of the specified ip ranges.
        If the any of the provided tags do not exist, they will be created.

        Args:
            ranges (list):
                A list of ip-range IDs. Should be UUIDs.
            tags (list):
                A list of tag annotation names to add to an ip-range.

        Returns:
            :obj:`boolean`:
                `True` if the ranges were tagged successfully, otherwise `False`.
        """
        payload = {"ipRangeIds": ranges, "tags": tags}
        return self._api.post(f"{V2_PREFIX}/ip-range/tag", json=payload)



from typing import Any
from xpanse.const import V1_PREFIX
from xpanse.endpoint import ExEndpoint
from xpanse.iterator import ExResultIterator


class ConnectorsServicesEndpoint(ExEndpoint):
    """
    Part of the Connectors V1 API.
    See: https://api.expander.expanse.co/api/v1/docs/
    """

    def list(self, **kwargs: Any) -> ExResultIterator:
        """
           This endpoint will return a paginated list of connector services.

           Args:
               limit (int, optional):
                   Returns at most this many results in a single api call.
                   Default is 100, max is 10000.
               pageToken (str, optional):
                   Page token for pagination.

        Returns:
            :obj:`ExResultIterator`:
                An iterator containing all of the connector services results. Results can be iterated
                or called by page using `<iterator>.next()`.

        Examples:
            >>> # Return all connector services dumped to a list:
            >>> bus =  client.connectors.services.list().dump()
        """
        return ExResultIterator(self._api, f"{V1_PREFIX}/connectors/services", kwargs)



from sys import exit
from csv import DictWriter

import click
from xpanse.client import ExClient


def flatten_rf(flow):
    """
    Flattens the Risky Flow objects into a flat dict to write to CSV.
    """
    return {
        "id": flow.get("id"),
        "businessUnit": flow.get("businessUnit", {}).get("name"),
        "riskRule": flow.get("riskRule", {}).get("name"),
        "internalAddress": flow.get("internalAddress"),
        "internalPort": flow.get("internalPort"),
        "externalAddress": flow.get("externalAddress"),
        "externalPort": flow.get("externalPort"),
        "flowDirection": flow.get("flowDirection"),
        "acked": flow.get("acked"),
        "protocol": flow.get("protocol"),
        "externalCountryCode": flow.get("externalCountryCode"),
        "internalCountryCode": flow.get("internalCountryCode"),
        "observationTimestamp": flow.get("observationTimestamp"),
        "created": flow.get("created"),
        "internalExposureTypes": ",".join(flow.get("internalExposureTypes")),
        "internalDomains": ",".join(flow.get("internalDomains")),
    }

@click.command()
@click.argument('file_name', type=click.File('w'))
@click.option('--created-before', 'created_before', default=None,
    help='Returns risky flows created before this timestamp. Formatted as YYYY-MM-DDTHH:MM:SS.mmmZ')
@click.option('--created-after', 'created_after', default=None,
    help='Returns risky flows created after this timestamp. Formatted as YYYY-MM-DDTHH:MM:SS.mmmZ')
@click.option('--tags', 'tags', default=None,
    help='Comma-separated string; Returns only results that are associated with the provided tags.')
@click.option('--ip', 'ip', default=None,
    help='Returns the risky flows that match the specified internal CIDR/IP Range/Address.')
def cli(file_name, created_before, created_after, tags, ip):

    # Initialize the Xpanse Client
    client = ExClient()

    # Filter and dump all Risky Flows into a list
    behavior = client.behavior.risky_flows.list(created_after=created_after,
                                                       created_before=created_before,
                                                       tag_names=tags,
                                                       internal_ip_range=ip).dump()

    # If we have o records returned, exit early
    if len(behavior) < 1:
        exit("0 Behavior flows returned for specified filters. No output file created.")

    # Extract the header fields by looking at the keys first dict in the list
    fields = list(flatten_rf(behavior[0]).keys())

    # Declare our CSV DictWriter instance
    writer = DictWriter(file_name, fieldnames=fields)

    # Writer row headers to file
    writer.writeheader()

    # For each flow collected, write it to the CSV file.
    for flow in behavior:
        writer.writerow(flatten_rf(flow))


if __name__ == "__main__":
    cli()



from typing import Any

from xpanse.const import V1_PREFIX
from xpanse.endpoint import ExEndpoint
from xpanse.iterator import ExResultIterator


class RiskRulesEndpoint(ExEndpoint):
    """
    Part of the Behvaior v1 API for access risk rules.
    See: https://api.expander.expanse.co/api/v1/docs/
    """

    def list(self, **kwargs: Any) -> ExResultIterator:
        """
        Returns the list of risk-rules matching the given criteria. Arguments should be passed as keyword args using
        the names below.

        Args:
            limit (int, optional):
                Returns at most this many results in a single api call.
            offset (int, optional):
                Returns results starting at this offset.
                Default is 0.
            business_unit (str, optional):
                Returns risk rules assigned to a specified Business Unit. This should
                be the Business Unit ID rather than the name.

        Returns:
            :obj:`ExResultIterator`:
                An iterator containing all risk rules results. Results can be iterated
                or called by page using `<iterator>.next()`.

        Examples:
            >>> # Prints all configured risk rules
            >>> rules = client.behavior.risk_rules.v1.list().dump()
            >>> print(rules)
        """
        arg_list = "?"
        if kwargs:
            for k, v in kwargs.items():
                if v is not None:
                    if k in ("limit", "offset"):
                        arg_list += f"page[{k}]={v}&"
                    else:
                        if isinstance(v, list):
                            arg_list += f"filter[{k.replace('_', '-')}]={','.join(v)}&"
                        else:
                            arg_list += f"filter[{k.replace('_', '-')}]={v}&"
        return ExResultIterator(
            self._api, f"{V1_PREFIX}/behavior/risk-rules{arg_list}", {}
        )



from typing import Any, Dict
from xpanse.const import V1_PREFIX
from xpanse.endpoint import ExEndpoint
from xpanse.iterator import ExResultIterator


class EntityIterator(ExResultIterator):
    """
    Iterator specifically for Entities which can be interacted with in the same way as the
    universal `ExResultIterator`.
    """

    def _get_data(self) -> Dict[str, Any]:
        """
        Returns the next page of data
        """
        if self._pages >= 1:
            resp = self._api.direct_get(self._next_url)
        else:
            resp = self._api.get(self._path, params=self._params)

        resp_as_json = resp.json()

        self._pages += 1
        self._next_url = resp_as_json.get("next", None)
        self._total = resp_as_json.get("count", 0)
        return resp_as_json["results"]


class EntitiesEndpoint(ExEndpoint):
    """
    Part of the Entities v1 API for accessing entities.
    See: https://api.expander.expanse.co/api/v1/docs/
    """

    def id_token(self):
        """
        A JWT is generated by default by this library is a Bearer token is provided. Call this endpoint will
        invalidate the current session and is not supported by this library.
        """
        raise NotImplementedError

    def list(self, **kwargs: Any) -> EntityIterator:
        """
        Returns the list of entities to which the authenticated user has access.

        Args:
            name (str, optional):
                Find an Entity by the given name
            parent (str, optional):
                Find Entities with the given parent Entity
            has_parent (boolean, optional):
                Find Entities that either have or do not have parents.

        Returns:
            :obj:`EntityIterator`:
                An iterator containing all of the entities results. Results can be iterated
                or called by page using `<iterator>.next()`.

        Examples:
            >>> # Print all entities with the name `Company X`
            >>> for ents in client.entities.entities.v1.list(name="Company X"):
            ...     for ent in ents:
            ...         print(ent)
        """
        return EntityIterator(self._api, f"{V1_PREFIX}/Entity/", kwargs)

    def get(self, id: str) -> Dict[str, Any]:
        """
        Returns the details for a given Entity.
        WARNING! Fetching IP Ranges from this endpoint is now deprecated and may return incorrect data.

        Args:
            id (str):
                ID for the entity. Should be a UUID.

        Returns:
            :obj:`dict`:
                A dictionary containing all of the details about an Entity.

        Examples:
            >>> # Returns an Entity
            >>> company_x = client.entities.entities.v1.get(<id>)
        """
        return self._api.get(f"{V1_PREFIX}/Entity/{id}/").json()



from typing import Any
from xpanse.const import V2_PREFIX
from xpanse.endpoint import ExEndpoint
from xpanse.iterator import ExResultIterator


class AssetEntitiesEndpoint(ExEndpoint):
    """
    Part of the Assets v2 API for handling Asset Entities.
    See: https://api.expander.expanse.co/api/v1/docs/
    """

    def list(self, **kwargs: Any) -> ExResultIterator:
        """
        Returns the list of Entities. Arguments should be passed as keyword args using
        the names below.

        Args:
            limit (int, optional):
                Returns at most this many results in a single api call.
                Default is 100, max is 10000.
            pageToken (str, optional):
                Returns results starting at this Page Token.

        Returns:
            :obj:`ExResultIterator`:
                An iterator containing all of the entities results. Results can be iterated
                or called by page using `<iterator>.next()`.

        Examples:
            >>>  # Return all entities and print each one:
            >>>  for entities in client.assets.entities.v2.list():
            ...     for entity in entities:
            ...     print(entity)
        """
        return ExResultIterator(self._api, f"{V2_PREFIX}/assets/entities", kwargs)



from xpanse.base_api import ExApi
from xpanse.api.assets.v2.account_integrations import AccountIntegrationsEndpoint
from xpanse.api.assets.v2.annotations import AnnotationsEndpoint
from xpanse.api.assets.v2.entities import AssetEntitiesEndpoint
from xpanse.api.assets.v2.certificates import CertificatesEndpoint
from xpanse.api.assets.v2.certificate_properties import CertificatePropertiesEndpoint
from xpanse.api.assets.v2.cloud_resources import CloudResourcesEndpoint
from xpanse.api.assets.v2.domains import DomainsEndpoint
from xpanse.api.assets.v2.ip_range import IpRangeEndpoint
from xpanse.api.assets.v2.ips import IpsEndpoint
from xpanse.api.assets.v2.providers import ProvidersEndpoint
from xpanse.api.assets.v2.regions import RegionsEndpoint
from xpanse.api.assets.v2.certificate_issuers import CertificateIssuersEndpoint
from xpanse.api.assets.v2.domain_registrars import DomainRegistrarsEndpoint


class AccountIntegrationsApi(ExApi, AccountIntegrationsEndpoint):
    def __init__(self, session):
        super().__init__(session)

    @property
    def current_version(self):
        return "v2"

    @property
    def v2(self):
        return AccountIntegrationsEndpoint(self._api)


class AnnotationsApi(ExApi, AnnotationsEndpoint):
    def __init__(self, session):
        super().__init__(session)

    @property
    def current_version(self):
        return "v2"

    @property
    def v2(self):
        return AnnotationsEndpoint(self._api)


class AssetEntitiesApi(ExApi, AssetEntitiesEndpoint):
    def __init__(self, session):
        super().__init__(session)

    @property
    def current_version(self):
        return "v2"

    @property
    def v2(self):
        return AssetEntitiesEndpoint(self._api)


class CertificatesApi(ExApi, CertificatesEndpoint):
    def __init__(self, session):
        super().__init__(session)

    @property
    def current_version(self):
        return "v2"

    @property
    def v2(self):
        return CertificatesEndpoint(self._api)


class CertificatePropertiesApi(ExApi, CertificatePropertiesEndpoint):
    def __init__(self, session):
        super().__init__(session)

    @property
    def current_version(self):
        return "v2"

    @property
    def v2(self):
        return CertificatePropertiesEndpoint(self._api)


class CloudResourcesApi(ExApi, CloudResourcesEndpoint):
    def __init__(self, session):
        super().__init__(session)

    @property
    def current_version(self):
        return "v2"

    @property
    def v2(self):
        return CloudResourcesEndpoint(self._api)


class DomainsApi(ExApi, DomainsEndpoint):
    def __init__(self, session):
        super().__init__(session)

    @property
    def current_version(self):
        return "v2"

    @property
    def v2(self):
        return DomainsEndpoint(self._api)


class IPRangeApi(ExApi, IpRangeEndpoint):
    def __init__(self, session):
        super().__init__(session)

    @property
    def current_version(self):
        return "v2"

    @property
    def v2(self):
        return IpRangeEndpoint(self._api)


class IpsApi(ExApi, IpsEndpoint):
    def __init__(self, session):
        super().__init__(session)

    @property
    def current_version(self):
        return "v2"

    @property
    def v2(self):
        return IpsEndpoint(self._api)


class ProvidersApi(ExApi, ProvidersEndpoint):
    def __init__(self, session):
        super().__init__(session)

    @property
    def current_version(self):
        return "v2"

    @property
    def v2(self):
        return ProvidersEndpoint(self._api)


class RegionsApi(ExApi, RegionsEndpoint):
    def __init__(self, session):
        super().__init__(session)

    @property
    def current_version(self):
        return "v2"

    @property
    def v2(self):
        return RegionsEndpoint(self._api)


class CertificateIssuersApi(ExApi, CertificateIssuersEndpoint):
    def __init__(self, session):
        super().__init__(session)

    @property
    def current_version(self):
        return "v2"

    @property
    def v2(self):
        return CertificateIssuersEndpoint(self._api)


class DomainRegistrarsApi(ExApi, DomainRegistrarsEndpoint):
    def __init__(self, session):
        super().__init__(session)

    @property
    def current_version(self):
        return "v2"

    @property
    def v2(self):
        return DomainRegistrarsEndpoint(self._api)


class AssetsApi(ExApi):
    @property
    def domain_registrars(self):
        return DomainRegistrarsApi(self._api)

    @property
    def certificate_issuers(self):
        return CertificateIssuersApi(self._api)

    @property
    def account_integrations(self):
        return AccountIntegrationsApi(self._api)

    @property
    def annotations(self):
        return AnnotationsApi(self._api)

    @property
    def certificates(self):
        return CertificatesApi(self._api)

    @property
    def certificate_properties(self):
        return CertificatePropertiesApi(self._api)

    @property
    def cloud_resources(self):
        return CloudResourcesApi(self._api)

    @property
    def domains(self):
        return DomainsApi(self._api)

    @property
    def entities(self):
        return AssetEntitiesApi(self._api)

    @property
    def ip_range(self):
        return IPRangeApi(self._api)

    @property
    def ips(self):
        return IpsApi(self._api)

    @property
    def providers(self):
        return ProvidersApi(self._api)

    @property
    def regions(self):
        return RegionsApi(self._api)



from typing import Any, Dict, List
from xpanse.const import V1_PREFIX
from xpanse.endpoint import ExEndpoint
from xpanse.iterator import ExResultIterator


class ServicesEndpoint(ExEndpoint):
    """
    Part of the Services V1 API.
    See: https://api.expander.expanse.co/api/v1/docs/
    """

    def list(self, **kwargs: Any) -> ExResultIterator:
        """
        This endpoint will return a paginated list of services.

        Args:
            limit (int, optional):
                Returns at most this many results in a single api call.
                Default is 100, max is 10000.
            pageToken (str, optional):
                Page token for pagination.
            queryId (str, optional):
                ID of previously stored query.
            providerId (str, optional):
                Comma-separated string; Returns only results that were found on the given providers.
                If not set, results will include anything regardless of provider status.
                Cannot be used with providerName_neq or providerId_neq.
            providerId_neq (str, optional):
                Comma-separated string; Returns only results that were found on providers other than the given ones.
                Cannot be used with providerName or providerId.
            providerName (str, optional):
                Comma-separated string; Returns only results that were found on the given providers.
                If not set, results will include anything regardless of provider status.
                Cannot be used with providerName_neq or providerId_neq.
            providerName_neq (str, optional):
                Comma-separated string; Returns only results that were found on providers other than the given ones.
                Cannot be used with providerName or providerId.
            businessUnitId (str, optional):
                Comma-separated string; Returns only results whose Business Unit's ID falls in the provided list.
                NOTE: If omitted, API will return results for all Business Units the user has permissions to view.
                Cannot be used with businessUnitId_neq or businessUnitName_neq.
            businessUnitId_neq (str, optional):
                Comma-separated string; Returns only results whose Business Unit's ID are other than the provided list.
                NOTE: If omitted, the API will return results for all Business Units the user has permissions to view.
                Cannot be used with businessUnitId or businessUnitName.
            businessUnitName (str, optional):
                Comma-separated string; Returns only results whose Business Unit's name falls in the provided list.
                NOTE: If omitted, API will return results for all Business Units the user has permissions to view.
                Cannot be used with businessUnitId_neq or businessUnitName_neq.
            businessUnitName_neq (str, optional):
                Comma-separated string; Returns only results whose Business Unit's Name are other than the provided list.
                NOTE: If omitted, the API will return results for all Business Units the user has permissions to view.
                Cannot be used with businessUnitId or businessUnitName.
            tagId (str, optional):
                Comma-separated string; Returns only results that are associated with the provided tags.
                The tag id should be used here rather than the tag name.
                Cannot be used with tagId_neq or tagName_neq.
            tagId_neq (str, optional):
                Comma-separated string; Returns any assets with a tagId other than the provided set.
                The tag id should be used here rather than the tag name.
                Cannot be used with tagId or tagName.
            tagName (str, optional):
                Comma-separated string; Returns only results that are associated with the provided tags.
                The tag name should be used here rather than the tag id.
                Cannot be used with tagId_neq or tagName_neq.
            tagName_neq (str, optional):
                Comma-separated string; Returns any assets with a tagName other than the provided set.
                The tag name should be used here rather than the tag id.
                Cannot be used with tagId or tagName.
            classificationId (str, optional):
                Comma-separated string; Returns records with the specified service classifications.
                Cannot be used with classificationId_neq.
            classificationId_neq (str, optional):
                Comma-separated string; Returns records with service classification other than the ones specified.
                Cannot be used with classificationId.
            ipSearch (str, optional):
                Search for records with ip values in the specified subnets.
                Cannot be used with ipSearch_neq.
            ipSearch_neq (str, optional):
                Search for records with ip values not in the specified subnets.
                Cannot be used with ipSearch.
            domainSearch (str, optional):
                Comma-separated string; Search for given domain values via substring match.
                Cannot be used with domainSearch_neq.
            domainSearch_neq (str, optional):
                Comma-separated string; Search for results with domains that do not match (by substring) these values.
                Cannot be used with domainSearch.
            contentSearch (str, optional):
                Search for assets via content match.
            countryCode (str, optional):
                Comma-separated string of ISO-3166 two character country codes;
                Returns any assets with an IP located in a country in the provided set.
                Cannot be used with countryCode_neq.
            countryCode_neq (str, optional):
                Comma-separated string of ISO-3166 two character country codes;
                Returns any assets with an IP located in a country other than in the provided set.
                Cannot be used with countryCode.
            activityStatus (str, optional):
                Comma-separated string; Returns only results whose activity status matches one of the given values.
                Allowed values are `Active` and `Inactive`.
                Cannot be used with activityStatus_neq.
            activityStatus_neq (str, optional):
                Comma-separated string; Returns only results whose activity status not matching one of the given values.
                Allowed values are `Active` and `Inactive`.
                Cannot be used with activityStatus.
            discoveryType (str, optional):
                Describes the way a service was discovered (ColocatedOnIp, DirectlyDiscovered).
                Cannot be used with discoveryType_neq.
            discoveryType_neq (str, optional):
                Describes the way a service was discovered (ColocatedOnIp, DirectlyDiscovered).
                Cannot be used with discoveryType.
            portNumber (str, optional):
                Comma separated string; Returns assets located on the specified port.
                Cannot be used with portNumber_neq.
            portNumber_neq (str, optional):
                Comma separated string; Returns assets located on a port other than the specified port(s).
                Cannot be used with portNumber.
            include (str, optional):
                Comma-separated string; Include the provided fields as part of the serialized result.
                Values can include (allClassificationDetails, latestClassificationDetails)
            cloudManagementStatus (str, optional):
                Comma-separated string; Returns only results whose cloud management status matches one of the given values.
                Cannot be used with cloudManagementStatus_neq.
                Allowed values are `NotApplicable`, `ManagedCloud`, `UnmanagedCloud`.
            cloudManagementStatus_neq (str, optional):
                Comma-separated string; Returns only services whose cloud management status is other than given value(s).
                Cannot be used with cloudManagementStatus.
                Allowed values are `NotApplicable`, `ManagedCloud`, `UnmanagedCloud`.
            hostingEnvironment (str, optional):
                Comma-separated string; Returns only services whose hosting environment matches one of the given value(s).
                Cannot be used with hostingEnvironment!.
                Allowed values are 'OnPrem', 'Cloud'.
            hostingEnvironment_neg (str, optional):
                Comma-separated string; Returns only services whose hosting environment is other than the given value(s).
                Cannot be used with hostingEnvironment.
                Allowed values are 'OnPrem', 'Cloud'.
            sort (str, optional):
                Sort by specified properties.

        Returns:
            :obj:`ExResultIterator`:
                An iterator containing all of the services results. Results can be iterated
                or called by page using `<iterator>.next()`.

        Examples:
            >>> # Return all services dumped to a list:
            >>> bus =  client.services.v1.services.list().dump()
        """
        return ExResultIterator(self._api, f"{V1_PREFIX}/services/services", kwargs)

    def count(self, **kwargs: Any) -> Dict[str, Any]:
        """
        This endpoint will return a paginated list of services.

        Args:
            limit (int, optional):
                Returns at most this many results in a single api call.
                Default is 100, max is 10000.
            pageToken (str, optional):
                Page token for pagination.
            queryId (str, optional):
                ID of previously stored query.
            providerId (str, optional):
                Comma-separated string; Returns only results that were found on the given providers.
                If not set, results will include anything regardless of provider status.
                Cannot be used with providerName_neq or providerId_neq.
            providerId_neq (str, optional):
                Comma-separated string; Returns only results that were found on providers other than the given ones.
                Cannot be used with providerName or providerId.
            providerName (str, optional):
                Comma-separated string; Returns only results that were found on the given providers.
                If not set, results will include anything regardless of provider status.
                Cannot be used with providerName_neq or providerId_neq.
            providerName_neq (str, optional):
                Comma-separated string; Returns only results that were found on providers other than the given ones.
                Cannot be used with providerName or providerId.
            businessUnitId (str, optional):
                Comma-separated string; Returns only results whose Business Unit's ID falls in the provided list.
                NOTE: If omitted, API will return results for all Business Units the user has permissions to view.
                Cannot be used with businessUnitId_neq or businessUnitName_neq.
            businessUnitId_neq (str, optional):
                Comma-separated string; Returns only results whose Business Unit's ID are other than the provided list.
                NOTE: If omitted, the API will return results for all Business Units the user has permissions to view.
                Cannot be used with businessUnitId or businessUnitName.
            businessUnitName (str, optional):
                Comma-separated string; Returns only results whose Business Unit's name falls in the provided list.
                NOTE: If omitted, API will return results for all Business Units the user has permissions to view.
                Cannot be used with businessUnitId_neq or businessUnitName_neq.
            businessUnitName_neq (str, optional):
                Comma-separated string; Returns only results whose Business Unit's Name are other than the provided list.
                NOTE: If omitted, the API will return results for all Business Units the user has permissions to view.
                Cannot be used with businessUnitId or businessUnitName.
            tagId (str, optional):
                Comma-separated string; Returns only results that are associated with the provided tags.
                The tag id should be used here rather than the tag name.
                Cannot be used with tagId_neq or tagName_neq.
            tagId_neq (str, optional):
                Comma-separated string; Returns any assets with a tagId other than the provided set.
                The tag id should be used here rather than the tag name.
                Cannot be used with tagId or tagName.
            tagName (str, optional):
                Comma-separated string; Returns only results that are associated with the provided tags.
                The tag name should be used here rather than the tag id.
                Cannot be used with tagId_neq or tagName_neq.
            tagName_neq (str, optional):
                Comma-separated string; Returns any assets with a tagName other than the provided set.
                The tag name should be used here rather than the tag id.
                Cannot be used with tagId or tagName.
            assigneeUsername (str, optional):
                Comma-separated string; Returns only results whose assignee's username matches one of the given usernames.
                Use "Unassigned" to fetch issues that are not assigned to any user.
            classificationId (str, optional):
                Comma-separated string; Returns records with the specified service classifications.
                Cannot be used with classificationId_neq.
            classificationId_neq (str, optional):
                Comma-separated string; Returns records with service classification other than the ones specified.
                Cannot be used with classificationId.
            ipSearch (str, optional):
                Search for records with ip values in the specified subnets.
                Cannot be used with ipSearch_neq.
            ipSearch_neq (str, optional):
                Search for records with ip values not in the specified subnets.
                Cannot be used with ipSearch.
            domainSearch (str, optional):
                Comma-separated string; Search for given domain values via substring match.
                Cannot be used with domainSearch_neq.
            domainSearch_neq (str, optional):
                Comma-separated string; Search for results with domains that do not match (by substring) these values.
                Cannot be used with domainSearch.
            contentSearch (str, optional):
                Search for assets via content match.
            countryCode (str, optional):
                Comma-separated string of ISO-3166 two character country codes;
                Returns any assets with an IP located in a country in the provided set.
                Cannot be used with countryCode_neq.
            countryCode_neq (str, optional):
                Comma-separated string of ISO-3166 two character country codes;
                Returns any assets with an IP located in a country other than in the provided set.
                Cannot be used with countryCode.
            activityStatus (str, optional):
                Comma-separated string; Returns only results whose activity status matches one of the given values.
                Allowed values are `Active` and `Inactive`.
                Cannot be used with activityStatus_neq.
            activityStatus_neq (str, optional):
                Comma-separated string; Returns only results whose activity status not matching one of the given values.
                Allowed values are `Active` and `Inactive`.
                Cannot be used with activityStatus.
            discoveryType (str, optional):
                Describes the way a service was discovered (ColocatedOnIp, DirectlyDiscovered).
                Cannot be used with discoveryType_neq.
            discoveryType_neq (str, optional):
                Describes the way a service was discovered (ColocatedOnIp, DirectlyDiscovered).
                Cannot be used with discoveryType.
            portNumber (str, optional):
                Comma separated string; Returns assets located on the specified port.
                Cannot be used with portNumber_neq.
            portNumber_neq (str, optional):
                Comma separated string; Returns assets located on a port other than the specified port(s).
                Cannot be used with portNumber.
            cloudManagementStatus (str, optional):
                Comma-separated string; Returns only results whose cloud management status matches one of the given values.
                Cannot be used with cloudManagementStatus_neq.
                Allowed values are `NotApplicable`, `ManagedCloud`, `UnmanagedCloud`.
            cloudManagementStatus_neq (str, optional):
                Comma-separated string; Returns only services whose cloud management status is other than given value(s).
                Cannot be used with cloudManagementStatus.
                Allowed values are `NotApplicable`, `ManagedCloud`, `UnmanagedCloud`.
            hostingEnvironment (str, optional):
                Comma-separated string; Returns only services whose hosting environment matches one of the given value(s).
                Cannot be used with hostingEnvironment!.
                Allowed values are 'OnPrem', 'Cloud'.
            hostingEnvironment_neg (str, optional):
                Comma-separated string; Returns only services whose hosting environment is other than the given value(s).
                Cannot be used with hostingEnvironment.
                Allowed values are 'OnPrem', 'Cloud'.

        Returns:
            :obj:`dict`:
                An dictionary containing services count information

        Examples:
            >>> # Return the count of all services
            >>> count =  client.services.v1.services.count()
        """
        return self._api.get(
            f"{V1_PREFIX}/services/services/count", params=kwargs
        ).json()

    def get(self, id: str, **kwargs: Any) -> Dict[str, Any]:
        """
        Returns the details for a given Service. Arguments should be passed as keyword args using
        the names below.

        Args:
            id (str):
                ID of the requested service.

        Returns:
            :obj:`dict`:
                A dictionary containing all of the details about the service.

        Examples:
            >>> # Return service.
            >>> service = client.services.services.get(<id>)
        """
        return self._api.get(
            f"{V1_PREFIX}/services/services/{id}", params=kwargs
        ).json()

    def csv(self, file: str, **kwargs: Any) -> bool:
        """
        This endpoint will export a filtered list of services to csv.

        Args:
            file (str):
                The name of the returned CSV file.
            limit (int, optional):
                Returns at most this many results in a single api call.
                Default is 100, max is 10000.
            pageToken (str, optional):
                Page token for pagination.
            queryId (str, optional):
                ID of previously stored query.
            providerId (str, optional):
                Comma-separated string; Returns only results that were found on the given providers.
                If not set, results will include anything regardless of provider status.
                Cannot be used with providerName_neq or providerId_neq.
            providerId_neq (str, optional):
                Comma-separated string; Returns only results that were found on providers other than the given ones.
                Cannot be used with providerName or providerId.
            providerName (str, optional):
                Comma-separated string; Returns only results that were found on the given providers.
                If not set, results will include anything regardless of provider status.
                Cannot be used with providerName_neq or providerId_neq.
            providerName_neq (str, optional):
                Comma-separated string; Returns only results that were found on providers other than the given ones.
                Cannot be used with providerName or providerId.
            businessUnitId (str, optional):
                Comma-separated string; Returns only results whose Business Unit's ID falls in the provided list.
                NOTE: If omitted, API will return results for all Business Units the user has permissions to view.
                Cannot be used with businessUnitId_neq or businessUnitName_neq.
            businessUnitId_neq (str, optional):
                Comma-separated string; Returns only results whose Business Unit's ID are other than the provided list.
                NOTE: If omitted, the API will return results for all Business Units the user has permissions to view.
                Cannot be used with businessUnitId or businessUnitName.
            businessUnitName (str, optional):
                Comma-separated string; Returns only results whose Business Unit's name falls in the provided list.
                NOTE: If omitted, API will return results for all Business Units the user has permissions to view.
                Cannot be used with businessUnitId_neq or businessUnitName_neq.
            businessUnitName_neq (str, optional):
                Comma-separated string; Returns only results whose Business Unit's Name are other than the provided list.
                NOTE: If omitted, the API will return results for all Business Units the user has permissions to view.
                Cannot be used with businessUnitId or businessUnitName.
            tagId (str, optional):
                Comma-separated string; Returns only results that are associated with the provided tags.
                The tag id should be used here rather than the tag name.
                Cannot be used with tagId_neq or tagName_neq.
            tagId_neq (str, optional):
                Comma-separated string; Returns any assets with a tagId other than the provided set.
                The tag id should be used here rather than the tag name.
                Cannot be used with tagId or tagName.
            tagName (str, optional):
                Comma-separated string; Returns only results that are associated with the provided tags.
                The tag name should be used here rather than the tag id.
                Cannot be used with tagId_neq or tagName_neq.
            tagName_neq (str, optional):
                Comma-separated string; Returns any assets with a tagName other than the provided set.
                The tag name should be used here rather than the tag id.
                Cannot be used with tagId or tagName.
            assigneeUsername (str, optional):
                Comma-separated string; Returns only results whose assignee's username matches one of the given usernames.
                Use "Unassigned" to fetch issues that are not assigned to any user.
            classificationId (str, optional):
                Comma-separated string; Returns records with the specified service classifications.
                Cannot be used with classificationId_neq.
            classificationId_neq (str, optional):
                Comma-separated string; Returns records with service classification other than the ones specified.
                Cannot be used with classificationId.
            ipSearch (str, optional):
                Search for records with ip values in the specified subnets.
                Cannot be used with ipSearch_neq.
            ipSearch_neq (str, optional):
                Search for records with ip values not in the specified subnets.
                Cannot be used with ipSearch.
            domainSearch (str, optional):
                Comma-separated string; Search for given domain values via substring match.
                Cannot be used with domainSearch_neq.
            domainSearch_neq (str, optional):
                Comma-separated string; Search for results with domains that do not match (by substring) these values.
                Cannot be used with domainSearch.
            contentSearch (str, optional):
                Search for assets via content match.
            countryCode (str, optional):
                Comma-separated string of ISO-3166 two character country codes;
                Returns any assets with an IP located in a country in the provided set.
                Cannot be used with countryCode_neq.
            countryCode_neq (str, optional):
                Comma-separated string of ISO-3166 two character country codes;
                Returns any assets with an IP located in a country other than in the provided set.
                Cannot be used with countryCode.
            activityStatus (str, optional):
                Comma-separated string; Returns only results whose activity status matches one of the given values.
                Allowed values are `Active` and `Inactive`.
                Cannot be used with activityStatus_neq.
            activityStatus_neq (str, optional):
                Comma-separated string; Returns only results whose activity status not matching one of the given values.
                Allowed values are `Active` and `Inactive`.
                Cannot be used with activityStatus.
            discoveryType (str, optional):
                Describes the way a service was discovered (ColocatedOnIp, DirectlyDiscovered).
                Cannot be used with discoveryType_neq.
            discoveryType_neq (str, optional):
                Describes the way a service was discovered (ColocatedOnIp, DirectlyDiscovered).
                Cannot be used with discoveryType.
            portNumber (str, optional):
                Comma separated string; Returns assets located on the specified port.
                Cannot be used with portNumber_neq.
            portNumber_neq (str, optional):
                Comma separated string; Returns assets located on a port other than the specified port(s).
                Cannot be used with portNumber.
            cloudManagementStatus (str, optional):
                Comma-separated string; Returns only results whose cloud management status matches one of the given values.
                Cannot be used with cloudManagementStatus_neq.
                Allowed values are `NotApplicable`, `ManagedCloud`, `UnmanagedCloud`.
            cloudManagementStatus_neq (str, optional):
                Comma-separated string; Returns only services whose cloud management status is other than given value(s).
                Cannot be used with cloudManagementStatus.
                Allowed values are `NotApplicable`, `ManagedCloud`, `UnmanagedCloud`.
            hostingEnvironment (str, optional):
                Comma-separated string; Returns only services whose hosting environment matches one of the given value(s).
                Cannot be used with hostingEnvironment!.
                Allowed values are 'OnPrem', 'Cloud'.
            hostingEnvironment_neg (str, optional):
                Comma-separated string; Returns only services whose hosting environment is other than the given value(s).
                Cannot be used with hostingEnvironment.
                Allowed values are 'OnPrem', 'Cloud'.
            sort (str, optional):
                Sort by specified properties.


        Returns:
            :obj:`boolean`:
                `True` if the download was successful, otherwise `False`.

        Examples:
            >>> # Download a csv of all services
            >>> client.services.services.csv(file="services.csv")
        """
        kwargs["filename"] = file
        return self._api.csv(
            path=f"{V1_PREFIX}/services/services/csv", file_=file, **kwargs
        )

    def updates(self, **kwargs: Any) -> ExResultIterator:
        """
        This endpoint will return a paginated list of services updates.

        Args:
            limit (int, optional):
                Returns at most this many results in a single api call.
                Default is 100, max is 10000.
            pageToken (str, optional):
                Page token for pagination.
            include (str, optional):
                Comma-separated string; Include the provided fields as part of the serialized result.
                `service` is the only allowed value and can be provided to include the service object in the update response.
            createdAfter (str, optional):
                Returns only results created after the provided timestamp (YYYY-MM-DDTHH:MM:SSZ).
            createdBefore (str, optional):
                Returns only results created before the provided timestamp (YYYY-MM-DDTHH:MM:SSZ).

        Returns:
            :obj:`ExResultIterator`:
                An iterator containing all of the service update results. Results can be iterated
                or called by page using `<iterator>.next()`.

        Examples:
            >>> # Return all services updates between two dates:
            >>> for res in client.services.v1.services.updates(createdAfter="2020-12-20T00:00:00Z", createdBefore="2020-12-21T00:00:00Z")
            ...     for service_update in res:
            ...         print(service_update)
        """
        return ExResultIterator(self._api, f"{V1_PREFIX}/services/updates", kwargs)



from typing import Any, Dict

from xpanse.const import V1_PREFIX
from xpanse.endpoint import ExEndpoint
from xpanse.iterator import ExResultIterator


class RiskyFlowsEndpoint(ExEndpoint):
    """
    Part of the Behvaior v1 API for access risky flows.
    See: https://api.expander.expanse.co/api/v1/docs/
    """

    def list(self, **kwargs: Any) -> ExResultIterator:
        """
        Returns the list of risky flows matching the given criteria. Arguments should be passed as keyword args using
        the names below.

        Args:
            created_after (string, optional):
                Returns risky flows that were created after this timestamp.
                Should be formatted as YYYY-MM-DDTHH:MM:SS.mmmZ
            created_before (string, optional):
                Returns risky flows that were created before this timestamp.
                Should be formatted as YYYY-MM-DDTHH:MM:SS.mmmZ
            limit (int, optional):
                Returns at most this many results in a single api call.
            offset (int, optional):
                Returns results starting at this offset.
                Default is 0.
            business_unit (str, optional):
                Returns risky flows for IPs which belong to the specified Business Unit. This should
                be the Business Unit ID rather than the name.
            risk_rule (string, optional):
                Returns risky flows that match the specified Risk Rule ID.
            tag_names (string, optional):
                Comma-separated string; Returns only results that are associated with the provided tags.
                The tag name should be used here rather than the tag id.
            internal_ip_range (string, optional):
                Returns the riksy flows that match the specified internal CIDR/IP Range/Address.
                Supported IP formats: a.b.c.d, a.b.c.d/e, a.b.c.d-a.b.c.d, a., a.*

        Returns:
            :obj:`ExResultIterator`:
                An iterator containing all of the risky flow results. Results can be iterated
                or called by page using `<iterator>.next()`.

        Examples:
            >>> # Prints all risky flow objects for a date range:
            >>> flows = client.behavior.risky_flows.v1.list(created_after="2020-12-20T00:00:00.000Z", created_before="2020-12-21T00:00:00.000Z").dump()
            >>> print(flows)
        """
        arg_list = "?"
        if kwargs:
            for k, v in kwargs.items():
                if v is not None:
                    if k in ("limit", "offset"):
                        arg_list += f"page[{k}]={v}&"
                    else:
                        if isinstance(v, list):
                            arg_list += f"filter[{k.replace('_', '-')}]={','.join(v)}&"
                        else:
                            arg_list += f"filter[{k.replace('_', '-')}]={v}&"
        return ExResultIterator(
            self._api, f"{V1_PREFIX}/behavior/risky-flows{arg_list}", {}
        )

    def get(self, risky_flow_id: str, **kwargs: Any) -> Dict[str, Any]:
        """
        Returns details about a single risky flow.

        Args:
            risky_flow_id (string):
                ID of a single risky flow.

        Returns:
            :obj:`dict`:
                A dict containing information about a specific risky flow.

        Examples:
            >>> # Print details of a risky flow
            >>> flow = client.behavior.risky_flows.v1.get(risky_flow_id="")
            >>> print(flow)
        """
        return self._api.get(
            f"{V1_PREFIX}/behavior/risky-flows/{risky_flow_id}", params=kwargs
        ).json()
